#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for in_rolls_fn.py

"""

import unittest
import pandas as pd
from naampy.in_rolls_fn import in_rolls_fn_gender, predict_fn_gender


class TestInRollsFn(unittest.TestCase):
    def setUp(self):
        names = [{"name": "yasmin"}, {"name": "vivek"}]
        self.f_names = ["nabha", "ayushmann"]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_in_rolls_fn(self):
        odf = in_rolls_fn_gender(self.df, "name")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.9)
        self.assertTrue(odf.iloc[1].prop_female < 0.1)

    def test_in_rolls_fn_state(self):
        odf = in_rolls_fn_gender(self.df, "name", "andaman")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.9)
        self.assertTrue(odf.iloc[1].prop_female < 0.1)

    def test_in_rolls_fn_state_year(self):
        odf = in_rolls_fn_gender(self.df, "name", "andhra", 1985)
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.9)
        self.assertTrue(odf.iloc[1].prop_female < 0.1)

    def test_in_rolls_fn_state_year_v1(self):
        odf = in_rolls_fn_gender(self.df, "name", "andhra", 1985, "v1")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.9)
        self.assertTrue(odf.iloc[1].prop_female < 0.1)

    def test_in_rolls_fn_state_year_v2(self):
        odf = in_rolls_fn_gender(self.df, "name", "andhra", 1985, "v2")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.9)
        self.assertTrue(odf.iloc[1].prop_female < 0.1)

    def test_predict_fn_gender(self):
        odf = predict_fn_gender(self.f_names)
        self.assertIn("pred_gender", odf.columns)
        self.assertTrue(odf.iloc[0].pred_gender == "female")
        self.assertTrue(odf.iloc[1].pred_gender == "male")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

import sys
import os
from os import path
import requests
from tqdm import tqdm


def isstring(s):
    # if we use Python 3
    if (sys.version_info[0] >= 3):
        return isinstance(s, str)
    # we use Python 2
    return isinstance(s, basestring)


def column_exists(df, col):
    """Check the column name exists in the DataFrame.

    Args:
        df (:obj:`DataFrame`): Pandas DataFrame.
        col (str): Column name.

    Returns:
        bool: True if exists, False if not exists.

    """
    if col and (col not in df.columns):
        print("The specify column `{0!s}` not found in the input file"
              .format(col))
        return False
    else:
        return True


def fixup_columns(cols):
    """Replace index location column to name with `col` prefix

    Args:
        cols (list): List of original columns

    Returns:
        list: List of column names

    """
    out_cols = []
    for col in cols:
        if type(col) == int:
            out_cols.append('col{:d}'.format(col))
        else:
            out_cols.append(col)
    return out_cols


def find_ngrams(vocab, text, n):
    """Find and return list of the index of n-grams in the vocabulary list.

    Generate the n-grams of the specific text, find them in the vocabulary list
    and return the list of index have been found.

    Args:
        vocab (:obj:`list`): Vocabulary list.
        text (str): Input text
        n (int): N-grams

    Returns:
        list: List of the index of n-grams in the vocabulary list.

    """

    wi = []

    if not isstring(text):
        return wi

    a = zip(*[text[i:] for i in range(n)])
    for i in a:
        w = ''.join(i)
        try:
            idx = vocab.index(w)
        except Exception as e:
            idx = 0
        wi.append(idx)
    return wi


def get_app_file_path(app_name, filename):
    user_dir = path.expanduser('~')
    app_data_dir = path.join(user_dir, '.' + app_name)
    if not path.exists(app_data_dir):
        os.makedirs(app_data_dir)
    file_path = path.join(app_data_dir, filename)
    return file_path


def download_file(url, target):

    headers = {}

    # Streaming, so we can iterate over the response.
    r = requests.get(url, stream=True, headers=headers)

    if r.status_code == 200:
        chunk_size = (64 * 1024)
        # Total size in bytes.
        total_size = int(r.headers.get('content-length', 0)) / chunk_size

        total_size += 1

        with open(target, 'wb') as f:
            for data in tqdm(r.iter_content(chunk_size), total=round(total_size, 1), unit_scale=chunk_size/1024, unit='KB'):
                f.write(data)
        return True
    else:
        print("ERROR: status_code={0:d}".format(r.status_code))
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for in_rolls_fn.py

"""

import unittest
import pandas as pd
from naampy.in_rolls_fn import in_rolls_fn_gender, predict_fn_gender


class TestInRollsFn(unittest.TestCase):
    def setUp(self):
        names = [{"name": "हेमा"}, {"name": "होशियार"}]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_in_rolls_fn(self):
        odf = in_rolls_fn_gender(self.df, "name", dataset="v2_native")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.8)
        self.assertTrue(odf.iloc[1].prop_female < 0.2)

    def test_in_rolls_fn_state(self):
        odf = in_rolls_fn_gender(self.df, "name", "up", dataset="v2_native")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.8)
        self.assertTrue(odf.iloc[1].prop_female < 0.2)

    def test_in_rolls_fn_state_year(self):
        odf = in_rolls_fn_gender(self.df, "name", "up", 1985, dataset="v2_native")
        self.assertIn("prop_female", odf.columns)
        self.assertTrue(odf.iloc[0].prop_female > 0.8)
        self.assertTrue(odf.iloc[1].prop_female < 0.2)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import pandas as pd

from pkg_resources import resource_filename

from utils import column_exists, fixup_columns, get_app_file_path, download_file


IN_ROLLS_DATA = {
    "v1": "https://github.com/appeler/instate/raw/main/data/instate_unique_ln_state_prop_v1.csv.gz",
}


class InRollsLnData:
    __df = None
    __model = None
    __year = None
    __tk = None
    __dataset = None

    @staticmethod
    def load_instate_data(dataset):
        data_fn = "instate_unique_ln_state_prop_{0:s}.csv.gz".format(dataset)
        data_path = get_app_file_path("instate", data_fn)
        if not os.path.exists(data_path):
            print("Downloading instate data from the server ({0!s})...".format(data_fn))
            if not download_file(IN_ROLLS_DATA[dataset], data_path):
                print("ERROR: Cannot download instate data file")
                return None
        else:
            print("Using cached instate data from local ({0!s})...".format(data_path))
        return data_path

    @classmethod
    def pred_last_state(cls, input):
        """
        Predict gender based on name
        Args:
            input (list of str): list of last name
        Returns:
            DataFrame: Pandas DataFrame with predictions
        """
        # load model
        if cls.__model is None:
            model_fn = resource_filename(__name__, "model")
            cls.__model = tf.keras.models.load_model(f"{model_fn}/instate_rmse")
        # create tokenizer
        if cls.__tk is None:
            cls.__tk = Tokenizer(num_words=None, char_level=True, oov_token="UNK")
            alphabet = "abcdefghijklmnopqrstuvwxyz"
            char_dict = {}
            for i, char in enumerate(alphabet):
                char_dict[char] = i + 1
            # Use char_dict to replace the tk.word_index
            cls.__tk.word_index = char_dict.copy()
            # Add 'UNK' to the vocabulary
            cls.__tk.word_index[cls.__tk.oov_token] = max(char_dict.values()) + 1

        input = [i.lower() for i in input]
        sequences = cls.__tk.texts_to_sequences(input)
        tokens = pad_sequences(sequences, maxlen=24, padding="post")

        results = cls.__model.predict(tokens)
        
        return pd.DataFrame(data={"name": input, "pred_state": results})

    @classmethod
    def last_state(cls, df, namecol, dataset="v1"):
        """Appends additional columns from state ratio data to the input DataFrame
        based on the last name.

        Removes the extra space. Checks if the name is the Indian electoral rolls data.
        If it is, outputs data from that row.

        Args:
            df (:obj:`DataFrame`): Pandas DataFrame containing the last name
                column.
            namecol (str or int): Column's name or location of the name in
                DataFrame.
            state (str): The state name of Indian electoral rolls data to be used.
                (default is None for all states)

        Returns:
            DataFrame: Pandas DataFrame with 31 additional columns

        """

        if namecol not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(namecol))
            return df

        df["__last_name"] = df[namecol].str.strip()
        df["__last_name"] = df["__last_name"].str.lower()

        if cls.__df is None or cls.__dataset != dataset:
            cls.__dataset = dataset
            data_path = InRollsLnData.load_instate_data(dataset)
            adf = pd.read_csv(data_path)
            cls.__df = adf
            cls.__df.rename(columns={"last_name": "__last_name"}, inplace=True)
        rdf = pd.merge(df, cls.__df, how="left", on="__last_name")

        return rdf

    @classmethod
    def state_to_lang(cls, df, statecolname):
        state_lang = pd.read_csv("data/state_to_languages.csv")
        res = df.merge(state_lang, left_on=statecolname, right_on='state', how='left')
        return(res)

    @staticmethod
    def list_states(dataset="v1"):
        data_path = InRollsLnData.load_instate_data(dataset)
        adf = pd.read_csv(data_path, usecols=["state"])
        return adf.state.unique()

last_state = InRollsLnData.last_state
pred_last_state = InRollsLnData.pred_last_state
state_to_lang = InRollsLnData.state_to_lang

